@@ -1,8 +1,13 @@
-//"use strict";
-
+// Copyright 2010 The Emscripten Authors.  All rights reserved.
+// Emscripten is available under two separate licenses, the MIT license and the
+// University of Illinois/NCSA Open Source License.  Both these licenses can be
+// found in the LICENSE file.
+//
 // Various tools for parsing LLVM. Utilities of various sorts, that are
 // specific to Emscripten (and hence not in utility.js).
 
+//"use strict";
+
 // Does simple 'macro' substitution, using Django-like syntax,
 // {{{ code }}} will be replaced with |eval(code)|.
 // NOTE: Be careful with that ret check. If ret is |0|, |ret ? ret.toString() : ''| would result in ''!
@@ -56,8 +61,12 @@ function preprocess(text, filenameHint) {
                 showStack.push(!(ident in this && this[ident] == value));
               } else if (op === '<') {
                 showStack.push(ident in this && this[ident] < value);
+              } else if (op === '<=') {
+                showStack.push(ident in this && this[ident] <= value);
               } else if (op === '>') {
                 showStack.push(ident in this && this[ident] > value);
+              } else if (op === '>=') {
+                showStack.push(ident in this && this[ident] >= value);
               } else {
                 error('unsupported preprocessor op ' + op);
               }
@@ -1460,14 +1469,14 @@ function heapAndOffset(heap, ptr) { // given   HEAP8, ptr   , we return    split
 }
 
 function makeEval(code) {
-  if (NO_DYNAMIC_EXECUTION == 1) {
+  if (DYNAMIC_EXECUTION == 0) {
     // Treat eval as error.
-    return "abort('NO_DYNAMIC_EXECUTION=1 was set, cannot eval');";
+    return "abort('DYNAMIC_EXECUTION=0 was set, cannot eval');";
   }
   var ret = '';
-  if (NO_DYNAMIC_EXECUTION == 2) {
+  if (DYNAMIC_EXECUTION == 2) {
     // Warn on evals, but proceed.
-    ret += "err('Warning: NO_DYNAMIC_EXECUTION=2 was set, but calling eval in the following location:');\n";
+    ret += "err('Warning: DYNAMIC_EXECUTION=2 was set, but calling eval in the following location:');\n";
     ret += "err(stackTrace());\n";
   }
   ret += code;
