@@ -1,3 +1,10 @@
+/*
+ * Copyright 2015 The Emscripten Authors.  All rights reserved.
+ * Emscripten is available under two separate licenses, the MIT license and the
+ * University of Illinois/NCSA Open Source License.  Both these licenses can be
+ * found in the LICENSE file.
+ */
+
 #ifndef __emscripten_threading_h__
 #define __emscripten_threading_h__
 
@@ -82,7 +89,6 @@ uint64_t emscripten_atomic_xor_u64(void/*uint64_t*/ *addr, uint64_t val); // Emu
 
 int emscripten_futex_wait(volatile void/*uint32_t*/ *addr, uint32_t val, double maxWaitMilliseconds);
 int emscripten_futex_wake(volatile void/*uint32_t*/ *addr, int count);
-int emscripten_futex_wake_or_requeue(volatile void/*uint32_t*/ *addr, int count, volatile void/*uint32_t*/ *addr2, int cmpValue);
 
 typedef union em_variant_val
 {
@@ -232,19 +238,6 @@ struct thread_profiler_block
 	char name[32];
 };
 
-#define EM_PROXIED_UTIME 12
-#define EM_PROXIED_UTIMES 13
-#define EM_PROXIED_CHROOT 37
-#define EM_PROXIED_FPATHCONF 46
-#define EM_PROXIED_CONFSTR 68
-#define EM_PROXIED_SYSCONF 72
-#define EM_PROXIED_ATEXIT 110
-#define EM_PROXIED_GETENV 111
-#define EM_PROXIED_CLEARENV 112
-#define EM_PROXIED_SETENV 113
-#define EM_PROXIED_UNSETENV 114
-#define EM_PROXIED_PUTENV 115
-#define EM_PROXIED_TZSET 119
 #define EM_PROXIED_PTHREAD_CREATE 137
 #define EM_PROXIED_SYSCALL 138
 
